/*
 * @file exo_ahw_vdp_multifeep_thruster.h
 *
 * @brief This file has vendor driver porting function prototype of thruster
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "exo_hal_common.h"
#include "exo_ahw_al_thruster_common.h"
#include "stdlib.h"
#include "exo_io_al_i2c_common.h"

#ifndef DRIVERS_EXO_HAL_DRIVER_FW_EXO_AHW_AL_DRIVER_API_EXO_AHW_AL_WRAPPER_MORPHEUS_MULTIFEEP_INC_EXO_AHW_VDP_MORPHEUS_MULTIFEEP_H_
#define DRIVERS_EXO_HAL_DRIVER_FW_EXO_AHW_AL_DRIVER_API_EXO_AHW_AL_WRAPPER_MORPHEUS_MULTIFEEP_INC_EXO_AHW_VDP_MORPHEUS_MULTIFEEP_H_

/**
 * @brief This function initializes the Thruster
 * @param[in] ahw_al_h_thruster - instance pointer of thruster
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_init(ahw_al_thruster_hdle *ahw_al_h_thruster);

/**
 * @brief This function is used to set the force in (micro Newton)
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[in] force - set the force
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_unit_set_frc(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,float force);

/**
 * @brief  This function is used to reset the system
 * @param[in] al_thruster - instance pointer of thruster
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_reset(ahw_al_thruster_hdle *al_thruster);

/**
 * @brief  This function is used to boot the system
 * @param[in] al_thruster - instance pointer of thruster
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_boot(ahw_al_thruster_hdle *al_thruster);

/**
 * @brief  This function is used to set the on/off status
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[in] on_off - set the  power ON and OFF
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_unit_set_pwr_on_off(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t on_off);

/**
 * @brief  This function is used to set the mode of thruster
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[in] mode - set the mode
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_unit_set_mode(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t mode);

/**
 * @brief  This function is used to set the mode of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts  ahw_vdp_thruster_set_mode(ahw_al_thruster_hdle *al_thruster,uint8_t system_id);

/**
 * @brief  This function is used to get the mode of the system
 * @param[in] al_thruster - in This stance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] mode - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the mode of the system
 */
hal_ret_sts ahw_vdp_thruster_get_mode(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *mode);

/**
 * @brief  This function is used to get the mode of thruster
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] mode - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the success or error code.
 */
hal_ret_sts ahw_vdp_thruster_unit_get_mode(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *mode);

/**
 * @brief  This function is used to get the status of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] info - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_sys_sts(ahw_al_thruster_hdle *al_thruster, uint8_t system_id,al_thruster_sts *info);

/**
 * @brief  This function is used to get the serial number of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] srl_num - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_sys_srl_num(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *srl_num);

/**
 * @brief  This function is used to get the housekeeping1 data
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] info - decoded frame is updated in this structure pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_housekeeping1_data(ahw_al_thruster_hdle *al_thruster,al_thruster_housekeeping1 *info);

/**
 * @brief  This function is used to get the housekeeping2 data
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] info - decoded frame is updated in this structure pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_housekeeping2_data(ahw_al_thruster_hdle *al_thruster,al_thruster_housekeeping2 *info,uint8_t system_id);

/**
 * @brief  This function is used to get the housekeeping3
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] housekeeping3 - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_housekeeping3_data(ahw_al_thruster_hdle *al_thruster,uint8_t *housekeeping3);

/**
 * @brief  This function is used to get the power of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] on_off - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the ON or OFF status of of the system
 */
hal_ret_sts ahw_vdp_thruster_unit_get_pwr_on_off(ahw_al_thruster_hdle *al_thruster,uint8_t *on_off,uint8_t system_id);

/**
 * @brief  This function is used to get the force in (micro N)
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id - id of the system
 * @param[out] frc - decoded frame is updated in this pointer variable
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_unit_get_frc(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *frc);

/**
 * @brief  This function is used to get the master time from housekeeping1 data
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] mst_tm - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_mst_time(ahw_al_thruster_hdle *al_thruster,uint32_t *mst_tm);

/**
 * @brief  This function is used to get the the backplane temperature
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] bckpln_temp - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_bckplane_temp(ahw_al_thruster_hdle *al_thruster,uint8_t* bckpln_temp);

/**
 * @brief  This function is used to get total force of thruster
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] bckpln_frc - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_bckplane_tot_frc(ahw_al_thruster_hdle *al_thruster,uint16_t* bckpln_frc);

/**
 * @brief  This function is used to get total azimuthal angle
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] tot_azmth - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_tot_azimth_ang(ahw_al_thruster_hdle *al_thruster,uint16_t* tot_azmth);

/**
 * @brief  This function is used to get the total polar angle
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] tot_plr - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_tot_polar_ang(ahw_al_thruster_hdle *al_thruster,uint8_t *tot_plr);

/**
 * @brief  This function is used to get the total power consumption
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] pwr_cnsp - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_tot_pwr_cnsp(ahw_al_thruster_hdle *al_thruster,uint16_t *pwr_cnsp);

/**
 * @brief  This function is used to get the unregulated voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] unreg_vol - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unreg_vol(ahw_al_thruster_hdle *al_thruster,uint8_t *unreg_vol);

/**
 * @brief  This function is used to get the remaining propellant
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] prplt - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_remn_prplt(ahw_al_thruster_hdle *al_thruster,uint16_t *prplt);

/**
 * @brief  This function is used to get the polar angle of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] polar - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unit_polar_ang(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *polar);

/**
 * @brief  This function is used to get the Azimuthal angle of system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] azimuthal - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unit_azimth_ang(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *azimuthal);

/**
 * @brief  This function is used to get the Power-Consumption of the system
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] power - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unit_pwr(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *power);

/**
 * @brief  This function is used to get the reset status
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] rst_sts - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_rst_sts(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *rst_sts);

/**
 * @brief  This function is used to get the total power
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] pwr - pointer to the data
 * @param[in] system_id -  id of the system
 * @retval success or error code - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_tot_pwr(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *pwr);

/**
 * @brief  This function is used to get the unregulated voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] unreg_vol - pointer to the data
 * @param[in] system_id -  id of the system
 * @retval hal_ret_sts- returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unreg_volt(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *unreg_vol);

/**
 * @brief  This function is used to get the ON status of the device
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] dev_sts - pointer to the data
 * @param[in] system_id -  id of the system
 * @retval thal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_dev_on_sts(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *dev_sts);

/**
 * @brief  This function is used to get the reset status
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] rst_sts - pointer to the data
 * @param[in] system_id -  id of the system
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unit_rst_sts(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *rst_sts);

/**
 * @brief  This function is used to get the board temperature
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] temp - pointer to the data
 * @param[in] system_id -  id of the system
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_brd_temp(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *temp);

/**
 * @brief  This function is used to get the remaining propellant
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] prplt - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_unit_prplt(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *prplt);

/**
 * @brief  This function is used to get the Unregulated voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] unreg_vol - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_ureg_volt(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *unreg_vol);

/**
 * @brief  This function is used to get the calibration value
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] calib - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_calib(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint32_t *calib);

/**
 * @brief  This function is used to get the heater error code
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] err_cde - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_heat_err_code(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *err_cde);

/**
 * @brief  This function is used to get the Duty cycle of the heater
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] duty_cycle - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_heat_duty_cycle(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *duty_cycle);

/**
 * @brief  This function is used to get the temperature of heater
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] temp - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_heat_temp(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,int8_t *temp);

/**
 * @brief  This function is used to get the target temperature of heater
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] trgt_temp - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_heat_trgt_temp(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,int8_t *trgt_temp);

/**
 * @brief  This function is used to get the emitter voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] emt_vol - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_emit_volt(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *emt_vol);

/**
 * @brief  This function is used to get the emitter current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] system_id -  id of the system
 * @param[out] emt_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_emit_cur(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *emt_cur);

/**
 * @brief  This function is used to get the emitter input current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] emt_ip_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_emit_inp_cur(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *emt_ip_cur);

/**
 * @brief  This function is used to get the extractor voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]ext_id -  id of the extractor
 * @param[in] system_id -  id of the system
 * @param[out] ext_vol - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_ext_volt(ahw_al_thruster_hdle *al_thruster,uint8_t ext_id,uint8_t system_id,uint16_t *ext_vol);

/**
 * @brief  This function is used to get the extractor current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]ext_id -  id of the extractor
 * @param[in] system_id -  id of the system
 * @param[out] ext_cur - pointer to the data
 * @retval hal_ret_sts- returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_ext_cur(ahw_al_thruster_hdle *al_thruster,uint8_t ext_id,uint8_t system_id,uint16_t *ext_cur);

/**
 * @brief  This function is used to get the extractor input current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]ext_id -  id of the extractor
 * @param[in] system_id -  id of the system
 * @param[out] ext_ip_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_ext_inp_cur(ahw_al_thruster_hdle *al_thruster,uint8_t ext_id,uint8_t system_id,uint16_t *ext_ip_cur);

/**
 * @brief  This function is used to get the neutralizer voltage
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]neut_id -  id of the neutralizer
 * @param[in] system_id -  id of the system
 * @param[out] neut_vol - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_neutlr_volt(ahw_al_thruster_hdle *al_thruster,uint8_t neut_id,uint8_t system_id,uint16_t *neut_vol);

/**
 * @brief  This function is used to get the neutralizer current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]neut_id -  id of the neutralizer
 * @param[in] system_id -  id of the system
 * @param[out] neut_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_neutlr_cur(ahw_al_thruster_hdle *al_thruster,uint8_t neut_id,uint8_t system_id,uint16_t *neut_cur);

/**
 * @brief  This function is used to get the neutralizer input current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]neut_id -  id of the neutralizer
 * @param[in] system_id -  id of the system
 * @param[out] neut_ip_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_neutlr_inp_cur(ahw_al_thruster_hdle *al_thruster,uint8_t neut_id,uint8_t system_id,uint16_t *neut_ip_cur);

/**
 * @brief  This function is used to get the of neutralizer return current
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in]neut_id -  id of the neutralizer
 * @param[in] system_id -  id of the system
 * @param[out] neut_ret_cur - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_neutlr_retn_cur(ahw_al_thruster_hdle *al_thruster,uint8_t neut_id,uint8_t system_id,uint16_t *neut_ret_cur);

/**
 * @brief  This function is used to get the major version number
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] mjr_vers_no - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_mjr_vers_no(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *mjr_vers_no);

/**
 * @brief  This function is used to get the minor version number
 * @param[in] al_thruster - instance pointer of thruster
 * @param[out] system_id -  id of the system
 * @param[out] mnr_vers_no - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_mnr_vers_no(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *mnr_vers_no);

/**
 * @brief  This function is used to get the patch version number
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] ptch_vers_no - pointer to the data
 * @retval hal_ret_sts - returns the  success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_patch_vers_no(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint16_t *ptch_vers_no);

/**
 * @brief  This function is used to get the board error code
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] err_cde - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_brd_err_cde(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *err_cde);

/**
 * @brief  This function is used to get the error counter
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  id of the system
 * @param[out] err_count - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_err_count(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint8_t *err_count);

/**
 * @brief  This function is used to get the start time
 * @param[in] al_thruster - instance pointer of thruster
 * @param[in] system_id -  structure pointer of response
 * @param[out] strt_time - pointer to the data
 * @retval hal_ret_sts - returns the success or error code
 */
hal_ret_sts ahw_vdp_thruster_get_strt_time(ahw_al_thruster_hdle *al_thruster,uint8_t system_id,uint32_t *strt_time);


#endif /* DRIVERS_EXO_HAL_DRIVER_FW_EXO_AHW_AL_DRIVER_API_EXO_AHW_AL_WRAPPER_MORPHEUS_MULTIFEEP_INC_EXO_AHW_VDP_MORPHEUS_MULTIFEEP_H_ */
