/**
 * @file tm_orig.c
 *
 * @brief This file contains functions that helps to create
 *        the telemetry frame
 *
 * The functions provided in the file helps to create telemetry
 * frame with crc and encryptions.
 *
 * @copyright Copyright 2023 Antaris, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*********************************************************************/
#include "tm_common.h"

/// Sequence number static global variable
static uint16 seq_no = 1;

/// s_crypt_params local instance to keep track of cryption parameters
static s_crypt_params tm_param_local;

/**
 * @brief This API initializes the s_tm_frame by setting all the values of it's
 *        members to zero.
 */
int8 tm_orig_init(s_tm_frame *tm_orig)
{
    int8 status;

    if(NULL != tm_orig)
    {
        os_memset(tm_orig, 0, sizeof(s_tm_frame));
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the start of frame passed in sof1 and sof2 value to
 * to the sof1 and sof2 members of passed in s_tm_frame type struct. Refer
 * section 3.2.2.1 in TC_TM_GS & Space Communication Protocol Design document
 * for more information
 */
int8 tm_orig_cfg_sof(s_tm_frame *tm_orig, uint8 sof1, uint8 sof2)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->sof1 = sof1;
        tm_orig->sof2 = sof2;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the tm_ctrl value to the tm_ctrl member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.2 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_tm_ctrl(s_tm_frame *tm_orig, uint8 tm_ctrl)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->tm_ctrl = tm_ctrl;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in seq_no_in value to the seq_no member
 * of passed in s_tm_frame type struct and also updates the global seq_no.
 * Refer section 3.2.2.4 in TC_TM_GS & Space Communication Protocol Design
 * document for more information
 */
int8 tm_orig_cfg_seq_no(s_tm_frame *tm_orig, uint16 seq_no_in)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->seq_no = seq_no_in;
        seq_no = seq_no_in;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in sat_id value to the sat_id member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.5 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_sat_id(s_tm_frame *tm_orig, uint16 sat_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->sat_id = sat_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in priority value to the qos member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.6 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_qos(s_tm_frame *tm_orig, uint8 priority)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->qos = priority;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in sa_id value to the sa_id member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.7 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_sa_id(s_tm_frame *tm_orig, uint16 sa_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->sa_id = sa_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in da_id value to the da_id member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.8 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_da_id(s_tm_frame *tm_orig, uint16 da_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->da_id = da_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in rm_id value to the rm_id member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.9 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_rm_id(s_tm_frame *tm_orig, uint8 rm_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->rm_id = rm_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in tm_id value to the tm_id member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.10 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_tm_id(s_tm_frame *tm_orig, uint16 tm_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->tm_id = tm_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in co_id (corelation id) value to
 * the tm_id member of passed in s_tm_frame type struct. Refer section
 * 3.2.2.15 in TC_TM_GS & Space Communication Protocol Design document
 * for more information
 */
int8 tm_orig_cfg_co_id(s_tm_frame *tm_orig, uint16 co_id)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->tm_co_id = co_id;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in tm_len value to the tm_len member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.11 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_tm_len(s_tm_frame *tm_orig, uint16 tm_len)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->tm_len = tm_len;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in value to the tm_len member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.12 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_tm_data(s_tm_frame *tm_orig, uint8 *data)
{
    int8 status;

    if((NULL != tm_orig) && (NULL != data))
    {
        tm_orig->tm_data = data;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in crc_8 value to the crc_8 member
 * of passed in s_tm_frame type struct. Refer section 3.2.2.13 in
 * TC_TM_GS & Space Communication Protocol Design document for more information
 */
int8 tm_orig_cfg_crc_8(s_tm_frame *tm_orig, uint8 crc_8)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->crc_8 = crc_8;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API stores the passed in end of frame (eof) to the eof member
 * member of passed in s_tm_frame type struct. Refer section 3.2.1.16
 * in TC_TM_GS & Space Communication Protocol Design document for more
 * more information
 */
int8 tm_orig_cfg_eof(s_tm_frame *tm_orig, uint8 eof)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->eof = eof;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API calculates hmac through the 3rd party library
 */
size_t tm_orig_compute_hmac(s_tm_frame *tm_orig, const uint8 *key,
                            const uint32 keysize, const uint32 datasize,
                            void *out)
{
    size_t status;

    if((NULL != tm_orig) && (NULL != key) && (NULL != out))
    {
        status =
            compute_hmac((const uint8 *)tm_orig, key, keysize, datasize, out);
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }
    return status;
}

/**
 * @brief This API stores the passed in hmac to the cmac_hmac member
 * of passed in s_tm_frame type struct
 */
int8 tm_orig_cfg_hmac(s_tm_frame *tm_orig, uint8 *hmac, size_t size)
{
    int8 status;

    if((NULL != tm_orig) && (NULL != hmac))
    {
        os_memcpy(tm_orig->cmac_hmac, hmac, size);
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API resets the sequence number to 1
 */
void tm_orig_reset_seq_no(void)
{
    seq_no = 1;
}

/**
 * @brief This API resets the timestamp to zero
 */
int8 tm_orig_reset_time_stamp(s_tm_frame *tm_orig)
{
    int8 status;

    if(NULL != tm_orig)
    {
        tm_orig->time_stamp = 0;
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API calculates the crc for the passed in input_str
 *         for the input size and stores the crc value to the val parameter
 *         equal to size input value.
 */
int8 tm_orig_compute_crc(const uint8 *input_str, uint8 *val, size_t size)
{

    int8 status;

    if((NULL != input_str) && (NULL != val))
    {
        *val = crc_8(input_str, size);
        status = TC_TM_LIB_OK;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API copies the structure data to the output buffer(dst). While
 * copying it takes care of the extension bit for the required members.
 * (Refer Table 3 in TC_TM_GS & Space Communication Protocol Design document
 * for more information) Finally it computes CRC ,hmac and embeds those value
 * to the final output buffer(dst).
 */
int32 tm_pack(uint8 *dst, s_tm_frame *tm_orig)
{
    int32 size = 0;
    int8 status = TC_TM_LIB_OK;
    size_t hmac_size = 0;

    if((NULL != dst) && (NULL != tm_orig))
    {
        os_memcpy(&dst[size], &tm_orig->sof1, sizeof(tm_orig->sof1));
        size += sizeof(tm_orig->sof1);

        os_memcpy(&dst[size], &tm_orig->sof2, sizeof(tm_orig->sof2));
        size += sizeof(tm_orig->sof2);

        os_memcpy(&dst[size], &tm_orig->tm_ctrl, sizeof(tm_orig->tm_ctrl));
        size += sizeof(tm_orig->tm_ctrl);

        tm_orig->time_stamp = get_utc_epoch();

        os_memcpy(&dst[size], &tm_orig->time_stamp,
                  sizeof(tm_orig->time_stamp));

        size += sizeof(tm_orig->time_stamp);

        // populate the sequence number to the structure
        tm_orig->seq_no = seq_no;

        os_memcpy(&dst[size], &tm_orig->seq_no, sizeof(tm_orig->seq_no));
        size += sizeof(tm_orig->seq_no);

        // Populate the value based on 1 or 2 byte encoding
        size += set_extnd_val_16(&tm_orig->sat_id, &dst[size],
                                 sizeof(tm_orig->sat_id));

        os_memcpy(&dst[size], &tm_orig->qos, sizeof(tm_orig->qos));
        size += sizeof(tm_orig->qos);

        // Populate the value based on 1 or 2 byte encoding
        size += set_extnd_val_16(&tm_orig->sa_id, &dst[size],
                                 sizeof(tm_orig->sa_id));

        // Populate the value based on 1 or 2 byte encoding
        size += set_extnd_val_16(&tm_orig->da_id, &dst[size],
                                 sizeof(tm_orig->da_id));

        os_memcpy(&dst[size], &tm_orig->rm_id, sizeof(tm_orig->rm_id));
        size += sizeof(tm_orig->rm_id);

        // Populate the value based on 1 or 2 byte encoding
        size += set_extnd_val_16(&tm_orig->tm_id, &dst[size],
                                 sizeof(tm_orig->tm_id));

        if(((tm_orig->tm_ctrl & TM_CTRL_CO_ID_BIT) >> TM_CTRL_CO_ID_POS) & 1)
        {
            os_memcpy(&dst[size], &tm_orig->tm_co_id,
                      sizeof(tm_orig->tm_co_id));
            size += sizeof(tm_orig->tm_co_id);
        }
        
        //tm_len is fixed as 2 bytes, before it is of variable lenght 1 or 2 bytes
        os_memcpy(&dst[size], &tm_orig->tm_len, sizeof(tm_orig->tm_len));
        size += sizeof(tm_orig->tm_len);


        if(((tm_orig->tm_ctrl & TM_CTRL_CRYPT_BIT) >> TM_CTRL_CRYPT_POS) & 1)
        {
            /* encrypt bit is set in tm_ctrl. encrypt the data */
            status = tm_orig_encrypt_data(&tm_param_local,
                                          (const uint8 *)tm_orig->tm_data,
                                          tm_orig->tm_len, tm_orig->tm_data);
        }

        if(TC_TM_LIB_OK == status)
        {
            os_memcpy(&dst[size], tm_orig->tm_data, tm_orig->tm_len);
            size += tm_orig->tm_len;
            /**
             *Either encryption is success or encryption bit is
             *not set in tm_ctrl
             */
            if(((tm_orig->tm_ctrl & TM_CTRL_AUTH_BIT) >> TM_CTRL_AUTH_POS) & 1)
            {
                hmac_size = compute_hmac(tm_orig->tm_data,
                                         (const uint8 *)TM_KEY, strlen(TM_KEY),
                                         tm_orig->tm_len, tm_orig->cmac_hmac);
                if(0 < hmac_size)
                {
                    os_memcpy(&dst[size], tm_orig->cmac_hmac, hmac_size);
                    size += hmac_size;
                }
                else
                {
                    /**
                     *compute_hmac failed for some reason. Set the size to
                     *compute_hmac status and return
                     */
                    size = hmac_size;
                    status = TC_TM_LIB_HMAC_ERROR;
                }
            }

            if(TC_TM_LIB_OK == status)
            {
                tm_orig_compute_crc(dst, &tm_orig->crc_8, size);

                os_memcpy(&dst[size], &tm_orig->crc_8, sizeof(tm_orig->crc_8));
                size += sizeof(tm_orig->crc_8);
                os_memcpy(&dst[size], &tm_orig->eof, sizeof(tm_orig->eof));
                size += sizeof(tm_orig->eof);
                seq_no++;
            }
            else
            {
                /**
                 * HMAC bit is set but the calculation failed.
                 * Size will already have the status of compute_hmac
                 */
            }
        }
        else
        {
            /**
             *encryption failed for some reason. Set the size to encryption
             *status and return
             */
            size = status;
        }
    }
    else
    {
        size = TC_TM_LIB_NULL_PTR;
    }

    return size;
}

/**
 * @brief This API copies the passed in encryption related parameters to the
 *        local structure. The parameters are utilised from the local structure
 *        during the encryption operation in tm_pack() api
 */
int8 tm_orig_select_encryption(s_crypt_params *param)
{
    int8 status = TC_TM_LIB_OK;

    if((NULL != param) && (NULL != param->iv) && (NULL != param->key))
    {
        os_memcpy(tm_param_local.iv, param->iv, 16);
        tm_param_local.key = param->key;
        tm_param_local.key_size = param->key_size;
        tm_param_local.encrypt_mode = param->encrypt_mode;
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }
    return status;
}

/**
 * @brief This API encrypts the passed in buffer of size "length" based on the
 *        input encryption "param" and stores the encrypted buffer to the
 *        "output" parameter
 */
int32 tm_orig_encrypt_data(s_crypt_params *param, const uint8 *buf,
                           int32 length, uint8 *output)
{
    int32 status;

    if((NULL != param) && (NULL != buf) && (NULL != param->key) &&
       (NULL != output))
    {
        status = aes_encrypt_data(param, buf, length, output);
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}

/**
 * @brief This API decrypts the passed in buffer of size "length" based on the
 *        input decryption "param" and stores the decrypted buffer to the
 *        "output" parameter
 */
int32 tm_orig_decrypt_data(s_crypt_params *param, const uint8 *buf,
                           size_t length, uint8 *output)
{
    int32 status;

    if((NULL != param) && (NULL != buf) && (NULL != param->key) &&
       (NULL != output))
    {
        status = aes_decrypt_data(param, buf, length, output);
    }
    else
    {
        status = TC_TM_LIB_NULL_PTR;
    }

    return status;
}
